/*******************************************************************************
File:			Control.c

Abstract:	Control subsystem function library.

Platform:	ATmega8

Functions:
extern void	CtrlRegInit(void);
extern void	CtrlSchedule(unsigned char schedule);
extern void	CtrlReport(void);
extern void	CtrlGetADC(unsigned char Hz);
extern viod	MainMonitor(void);
extern void	TenHertzMonitor(void);
extern void	OneSecondMonitor(void);
extern void	CtrlIdle(unsigned char schedule)

Versions:	01/14/10	GND	Gary N. Dion
*******************************************************************************/

// OS headers
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <avr/pgmspace.h>

// General purpose include files
#include "StdDefines.h"

// App required include files
#include	"Control.h"
#include "Serial.h"
#include "VFD.h"
#include "Messaging.h"
#include "EEPROM.h"

// Local definitions
#define PLAYLISTS	(6)						// Number of playlists on router
#define DEFAULT	(0)						// One of the screens displayed
#define PLAYLIST	(1)						// One of the screens displayed
#define VOLUME		(2)						// One of the screens displayed
#define DIMMER		(3)						// One of the screens displayed
#define WRAP		(4)						// Four screens are defined
#define NAME_LIMIT (140)					// Character limit for name buffer
#define TITLE_LIMIT (140)					// Character limit for title buffer

// Local variables
unsigned char activity = 0;				// Set on activity from the router
unsigned char active_screen = 0;			// Keeps track of our active screen
unsigned char screen_timeout = 0;		// Set on changing the screen
unsigned char outgoing_message = 0;		// Set when message is pending
unsigned char playlist = 1;				// Keeps track of current playlist
unsigned char dim = 0;						// Dim level for the display
unsigned char power = 0;					// State of power for the box
unsigned char power2 = 0;					// Added to make power sequencing easier
unsigned char blank = 0;					// Added to blank the screen
unsigned char name_size = 0;				// Size of Name
unsigned char title_size = 0;				// Size of Title
unsigned char new_name = 1;				// Set when new Name is received
unsigned char new_title = 1;				// Set when new Title is received
char vol[4];									// Buffer for holding volue level
char name[NAME_LIMIT];						// Buffer for stream Name
char title[TITLE_LIMIT];					// Buffer for stream Title

// Local function prototypes
static void	MainMonitor(void);
static void	FourHertzMonitor(void);
static void	OneSecondMonitor(void);


/******************************************************************************/
extern void		CtrlRegInit(void)
/*******************************************************************************
* ABSTRACT:	Initializes all I/O control registers.
*
* INPUT:		None
* RETURN:	Void
*/
{
// PORT C - Power control port for the display
// Bit/Pin 0 = Power Enable line for the display: 1 is on, 0 is off
	PORTC = 0x00;								// Initially off
	DDRC  = 0x41;								// Two outputs

// PORT B - Input port for the infrered receive line
// Bit/Pin 0 = Input for received IR data from the Onkyo TX-8255
	PORTB = 0x00;								// No pullups enabled
	DDRB  = 0x00;								// All lines are inputs

// PORT D - I/O port for serial comms and panel interface
// Bit/Pin 0 = USART RXD (connect to the TX line coming from the Asus router)
// Bit/Pin 1 = USART TXD (connect to router RX through 1.8/3.6k voltage divider)
// Bit/Pin 2 = Phase A of the rotary encoder (ground to enable)
// Bit/Pin 3 = Phase B of the rotary encoder (ground to enable)
// Bit/Pin 4 = "Push" input for the rotary encoder (ground to enable)
// Bit/Pin 5 = RXD for the software UART (but in this case, the power button!)
// Bit/Pin 6 = TXD for the software UART
	PORTD = 0x3C;								// Enable pullups for the four inputs
	DDRD = 0x40;								// TXD for the software UART is an output

	power = 0;									// Finish initialization in off "state"

	return;

}		// End CtrlRegInit(...)


/******************************************************************************/
extern void	CtrlSchedule(unsigned char schedule)
/*******************************************************************************
* ABSTRACT:	This code is called at 100Hz.  Schedule is the "frame" number
*				representing the segment of time within the second's period.
*
* INPUT		schedule		Schedule count 0-99.
* RETURN:	Void
*/
{
	MainMonitor();								// This -always- runs first

	if((schedule) % 10 == 0) 				// This then runs at 10Hz.
	{
		// Does not do anything in this project yet
	}

	if((schedule + 1) % 25 == 0) 			// This then runs at 4Hz.
	{
		FourHertzMonitor();
	}

	if((schedule + 50) % 100 == 0)		// This runs once a second
	{
		OneSecondMonitor();
	}

	return;

}		// End CtrlSchedule(...)


/******************************************************************************/
static void	MainMonitor(void)
/*******************************************************************************
* ABSTRACT:	Executed each cycle of the main loop.
*
* INPUT		Void
* RETURN:	Void
*/
{
	// Does not do anything in this project yet

	return;

}		// End MainMonitor(...)


/******************************************************************************/
static void	FourHertzMonitor(void)
/*******************************************************************************
* ABSTRACT:	This executes every fourth cycle of the main loop.
*
* INPUT:		None
* RETURN:	Void
*/
{
	unsigned char i;							// Generic loop counter variable
	static unsigned char name_leftright = 1;	// Scroll direction for Name
	static unsigned char name_offset = 0;		// The characters offset for Name
	static unsigned char title_leftright = 1;	// Scroll direction for Title
	static unsigned char title_offset = 0;		// The characters offset for Title

	if (active_screen == PLAYLIST)		// Showing playlist screen?
	{
		VFDClear();								// Clear display buffer
		VFDsendEEPROMString((unsigned int)EE_playlist, 0, 20); // "Playlist:" text
		VFDChar(playlist + '0');			// Print playlist number
	}
	else if (active_screen == VOLUME)	// Showing Volume Screen?
	{
		VFDClear();								// Clear display buffer
		VFDsendEEPROMString((unsigned int)EE_volume, 20, 20);	// "Volume:" text
		VFDsendString(vol, 28, 3);			// Show volume level
		VFDChar('%');							// And '%' sign
	}
	else if (active_screen == DIMMER)	// Showing Dimmer screen?
	{
		VFDClear();								// Clear display buffer
		VFDsendEEPROMString((unsigned int)EE_dimmer, 40, 20);	// D"immer:" text
		VFDChar(dim + '0');					// Show dimmer level
	}
	else if (power)							// Otherwise, if the power is on
	{
		VFDCursor(20);							// Move cursor to the second line
		for (i = 0; i < 20; i++)
		{
			VFDChar(' ');						// And clear out 20 characters
		}

		if ((new_name) || (name_size < 21))	// New Name just received?
		{
			VFDsendString(name, 20, 20);	// Just show it
			new_name = FALSE;					// And it's no longer new
			name_leftright = 1;				// Next time, scroll left
			name_offset = 0;					// And start scrolling with no offset
		}
		else										// So it's not a new Name...
		{
			if (name_leftright)				// Scrolling left?
			{
				name_offset++;					// Offset the text a little left
				VFDsendString(name + name_offset, 20, 20);	// Print it
				if (name_offset == name_size - 20) name_leftright = 0; // Reverse
			}
			else									// Scrolling right...
			{
				name_offset--;					// Offset the text a little right
				VFDsendString(name + name_offset, 20, 20);	// Print it
				if (name_offset == 0) name_leftright = 1;	// Reverse at end
			}

		}

		VFDCursor(40);							// Move cursor to the third line
		for (i = 0; i < 40; i++)
		{
			VFDChar(' ');						// And clear out 40 characters
		}

		if ((new_title) || (title_size < 41))  // New or short Title?
		{
			VFDsendString(title, 40, 40);	// Just show it
			new_title = FALSE;				// Now it's no longer new
			title_leftright = 1;				// Start scrolling left
			title_offset = 0;					// ... with no offset at first
		}
		else										// So it's not new or short
		{
			if (title_leftright)				// If scrolling left,
			{
				title_offset++;				// Scroll a little
				VFDsendString(title + title_offset, 40, 40);	// Show
				if (title_offset == title_size - 40) title_leftright = 0; // Reverse
			}
			else									// Scrolling right
			{
				title_offset--;				// Scroll a little
				VFDsendString(title + title_offset, 40, 40);	// Show the Title
				if (title_offset == 0) title_leftright = 1;	// Reverse again
			}

		}

	}

	if (blank) VFDClear();					// Blank screen if true
	
	if (power2) VFDRedraw();				// Redraw if power sequence complete

	if (outgoing_message)					// Message waiting to send the router?
	{
		switch(outgoing_message)
		{
			case(1):
				SerSendEEPROMString((unsigned int)EE_next, 20, 0);	// "mpc next"
				new_name = new_title = TRUE;
				break;
			case(2):
				SerSendEEPROMString((unsigned int)EE_prev, 20, 0);	// "mpc prev"
				new_name = new_title = TRUE;
				break;
			case(5):
				SerSendEEPROMString((unsigned int)EE_vup, 20, 0);// "mpc volume +10"
				break;
			case(6):
				SerSendEEPROMString((unsigned int)EE_vdn, 20, 0);// "mpc volume -10"
				break;
			case(3):
				if (++playlist > PLAYLISTS) playlist = 1;		// Increment and wrap
				SerSendEEPROMString((unsigned int)EE_load, 30, 0);	// "mpc load"
				SerSendByte(playlist + '0',0);				// Send playlist value
				SerSendByte(0x0d,0);								// And a carriage return
				SerSendEEPROMString((unsigned int)EE_play, 20, 0);	// "mpc next"
				break;
			case(4):
				if (--playlist == 0) playlist = PLAYLISTS;	// Decrement and wrap
				SerSendEEPROMString((unsigned int)EE_load, 30, 0);	// "mpc load"
				SerSendByte(playlist + '0',0);				// Send playlist value
				SerSendByte(0x0d,0);								// And a carriage return
				SerSendEEPROMString((unsigned int)EE_play, 20, 0);	// "mpc next"
				break;
		}

			outgoing_message = 0;			// Clear outgoing message
	}

	return;

}		// End FourHertzMonitor(...)


/******************************************************************************/
static void	OneSecondMonitor(void)
/*******************************************************************************
* ABSTRACT:	This code is called once per second.
*
* INPUT:		None
* RETURN:	Void
*/
{
	static unsigned char seconds = 0;	// Countdown timer for sequencing display

	if (power)									// If the power is on
	{
		if (seconds == 5) PORTC = 0x01;	// First, turn on the display power

		if (seconds == 4)	VFDInit();		// Then, initialize it

		if (seconds == 3)	VFDDim(dim);	// Then, command the dimming value

		if (seconds == 2)	
		{
			power2 = 1;							// Used in FourHertzMonitor()
			blank = 0;							// Ditto
		}

		// And finally, send "mpc play" to the router.
		if (seconds == 1)	SerSendEEPROMString((unsigned int)EE_play, 20, 0);

		if (activity)		// If we recently received signs of life from the router
		{
			activity--;							// Forget a little
		}
		else if (!(seconds))					// Unless we just now powered up
		{
			VFDClear();							// Show the user a timeout has occured
			VFDsendEEPROMString((unsigned int)EE_waiting, 0, 20);	// "Waiting..."
		}

	}
	else											// Otherwise, power is off
	{
		PORTC = 0x00;							// So power down the display
		VFDClear();								// And clear its buffer
		SerSendEEPROMString((unsigned int)EE_stop, 20, 0);	// "mpc stop" to router
		power2 = 0;								// Tell FourHertzMonitor()
		seconds = 6;							// And finally reset the startup timer
	}

	if (seconds) seconds--;					// Decrement seconds, if any

	if (screen_timeout) screen_timeout--;	// Decrement this too, and at zero
	else active_screen = DEFAULT;				// Change the active screen back

	return;

}		// End OneSecondMonitor(...)


extern void CtrlProcess(char *buffer, char message_type, unsigned char size)
/*******************************************************************************
* ABSTRACT:	Process a valid message.  This is called by MsgHandler0.
*
* INPUT:		buffer			Pointer to a buffer containing the new text
*				message_type	Type of text we're processing
*				size				"sizeof" the message text
* OUTPUT:	None.
*/
{
	unsigned char i;							// Generic loop counter variable

	if (message_type == VOLUM)				// If it's a Volume report
	{
		for (i = 0; i < 3; i++)				// For all three bytes,
		{
			vol[i] = buffer[i];				// Copy it into our buffer
		}

	}

	if ((active_screen) || !(power)) return;	// Exit if not DEFAULT or no power

	switch (message_type)
	{
		case (PROMPT):							// Doesn't do anything.
			break;

		case (STREAM):							// Just received something like "#7/10"
			VFDsendEEPROMString((unsigned int)EE_playing, 0, 20);	// "Playing"
			VFDsendString(buffer, 8, 10);	// Show the text w ereceived
			break;

		case (ELAPSED):						// Just received something like "5:36"
			VFDCursor(13);						// At this cursor location,
			for (i = 0; i < 8; i++)			// For the next eight locations,
			{
				VFDChar(' ');					// Clear them
			}

			VFDsendString(buffer, 13, 20);// And print the time in their place
			break;

		case (NAME):					// Just received something like "SLAY Radio"
			activity = 10;				// Let's remember the router is alive and well
			if (name_size != size) new_name = TRUE;	// Size change = new name

			if (size > NAME_LIMIT) size = NAME_LIMIT;	// No buffer overflows

			for (i = 0; i < size; i++)		// For each byte
			{
				name[i] = buffer[i];			// Copy to buffer
			}
			
			for (i = size; i < 20; i++)
			{
				name[i] = ' ';
			}


			name_size = size;					// Remember for next time
			break;

		case (TITLE):
			if (title_size != size) new_title = TRUE;	// Size change = new name

			if (size > TITLE_LIMIT) size = TITLE_LIMIT;	// No buffer overflows

			for (i = 0; i < size; i++)		// For each byte
			{
				title[i] = buffer[i];		// Copy to buffer
			}
			
			for (i = size; i < 40; i++)
			{
				title[i] = ' ';
			}


			title_size = size;				// Remember for next time
			break;

	}

	return;

}		// End CtrlProcess(...)


/******************************************************************************/
extern void	CtrlIdle(void)
/*******************************************************************************
* ABSTRACT:	This function is run repeatedly while the system is idle...
*
* INPUT:		None
* RETURN:	Void
*/
{
	// Does not do anything in this project yet

	return;

}		// End CtrlIdle(...)


/******************************************************************************/
extern void	CtrlRI(void)
/*******************************************************************************
* ABSTRACT:	This function is run at 19.2kHz and polls the infrared receiver
*				signal coming from the stereo receiver.  These decoded values may
*				not correspond to "actual" Onkyo command values, but the important
*				thing is within the context of our design, they are unique.
*
* INPUT:		None
* RETURN:	Void
*/
{
	unsigned char 			input_new;		// New port status
	static unsigned char input_old;		// Old port status
	static unsigned char bits = 0;		// Counter for the bits in the command
	static unsigned char count = 255;	// Counts between rising edges
	static unsigned short command;		// Decoded command

	input_new = (PINB & 0x01);				//	Grab new state of input
	if (input_new != input_old)			// Has anything changed?
	{
		if (input_new == 0x01)				// Only care about rising edge
		{
			if (count == 255)					// Big number = rising edge of 3ms pulse
			{
				command = 0;					// Start with a clean command
				bits = 0;						// And zero out our bit count
			}
			else
			{
				if ((count < 33) || (count > 82)) count = 255; // Too long or short
				else
				{
					if ((count > 53) && (count < 63)) // Between these pulse lengths
					{
						command |= 0x01;		// Let's count the bit as a one
					}
					if (count > 72)			// This catches the first long pulse
					{
						command |= 0x01;		// Which I also count bit as a one
					}

					// If we're here, 33 <= count >= 53.  Count bit as a zero.
					command <<= 1;				// Just shift the command
					bits++;						// And increment bit counter

					if (bits == 13)			// Ok, once we have 13 bits.
					{
						switch (command)		// Switch on the command received
						{
							case(0x2360):
								active_screen = DIMMER;	// Show a new screen
								screen_timeout = 2;		// Screen timeout = 2 seconds
								dim = 4;						// Brightish
								VFDDim(dim);				// Change the display
								break;
							case(0x2362):
								active_screen = DIMMER;	// Ditto
								screen_timeout = 2;
								dim = 2;						// Dimmer
								VFDDim(dim);
								break;
							case(0x2364):
								active_screen = DIMMER;	// Seeing a pattern here?
								screen_timeout = 2;
								dim = 0;
								VFDDim(dim);
								break;
							case(0x2B9C):
								active_screen = PLAYLIST;	// Then I'll stop
								screen_timeout = 5;
								outgoing_message = 3;		// Send message to the router
								break;
							case(0x2B9E):
								active_screen = PLAYLIST;
								screen_timeout = 5;
								outgoing_message = 4;
								break;
							case(0x2B98):
								active_screen = VOLUME;
								screen_timeout = 5;
								outgoing_message = 5;
								break;
							case(0x2B9A):
								active_screen = VOLUME;
								screen_timeout = 5;
								outgoing_message = 6;
								break;
							case(0x3E3A):
								active_screen = DEFAULT;
								outgoing_message = 1;
								break;
							case(0x3E3C):
								active_screen = DEFAULT;
								outgoing_message = 2;
								break;
							case(0x3E36):
								power = ON;		// Cool!  A "power on" command!
								break;
							case(0x3E38):
								power = OFF;	// Awww... KTHXBY
								break;
							case(0x3EB8):
								blank = OFF;	// Show the screen
								break;
							case(0x3E0C):
								blank = ON;		// Blank the screen
								break;
						}

					}

				}

			}

			count = 0;							// Reset count[er] for next edge
		}

	}

	if (count != 255) count++;				// Increment but cap count at 255

	input_old = input_new;					// And remember new state for next time

	return;

}		// End CtrlRI(...)


/******************************************************************************/
extern void		CtrlDebounce(void)
/*******************************************************************************
* ABSTRACT:	This function called at 19.2kHz from SIG_OVERFLOW0 in main.c.
*
* INPUT:		None
* RETURN:	Void
*/
{
	unsigned char 			inputs_new;		// New port status
	static unsigned char inputs_old;		// Old port status
	unsigned char 			spinner_new;	// New encoder value
	static unsigned char spinner_old;	// Old encoder value
	static unsigned char good_count;		// Stable samples, for debouncing

	inputs_new = (PIND & 0x3C);			//	Grab new state of inputs
	if (inputs_new != inputs_old)			// Have any bits changed?
	{
		good_count = 0;						// If so, reset the good counter
		inputs_old = inputs_new;			// And remember new state for mext time
	}
	else											// Same samples as last time
	{
		if (good_count < 10) good_count++;	// So increment and cap good_count
	}

	if (good_count == 5)						// This is true only once each change
	{
		if (!(inputs_new & 0x20))			// If the power button is pressed
		{
			if (power == ON) power = OFF;	//	Toggle power
			else power = ON;
		}

		if (!(inputs_new & 0x10))			// If the knob is pressed
		{
			active_screen++;					// Increment the active screen
			if (active_screen == WRAP) active_screen = 0;	// And wrap around
		}

		screen_timeout = 5;					// Set screen timeout to five seconds

		// This is a sneaky way to compare new and old encoder values
		spinner_new = ((spinner_old & 0x0C)<<4) | (inputs_new & 0x0C);

		switch (active_screen)				// Base effect on what screen is active
		{
			case(DEFAULT):
				switch (spinner_new)			// Based on new and old encoder values
				{
					case (0x08):			// These correspond to clockwise values
//					case (0x40):
					case (0xC4):
//					case (0x8C):				// Some position are between knob detents
						outgoing_message = 1;
						break;
//					case (0x04):			// These correspond to anticlockwise values
					case (0x4C):
//					case (0xC8):
					case (0x80):
						outgoing_message = 2;
						break;
				}
				break;
			case(PLAYLIST):
				switch (spinner_new)			// Base change on new and old values.
				{
					case (0x08):
//					case (0x40):
					case (0xC4):
//					case (0x8C):
						outgoing_message = 3;
						break;
//					case (0x04):
					case (0x4C):
//					case (0xC8):
					case (0x80):
						outgoing_message = 4;
						break;
				}
				break;
			case(VOLUME):
			case(12):
				switch (spinner_new)			// Base change on new and old values.
				{
					case (0x08):
//					case (0x40):
					case (0xC4):
//					case (0x8C):
						outgoing_message = 5;
						break;
//					case (0x04):
					case (0x4C):
//					case (0xC8):
					case (0x80):
						outgoing_message = 6;
						break;
				}
				break;
			case(DIMMER):
				switch (spinner_new)			// Base change on new and old values.
				{
					case (0x08):
//					case (0x40):
					case (0xC4):
//					case (0x8C):
						if (++dim == 8) dim = 0;	// Increment screen brightness
						break;
//					case (0x04):
					case (0x4C):
//					case (0xC8):
					case (0x80):
						if (--dim == 255) dim = 7;	// Decrement screen brightness
						break;
				}

				VFDDim(dim);					// Update the screen
				break;

		}

		spinner_old = spinner_new;			// Remember the old encoder value
	}

	return;

}		// End CtrlDebounce(...)
