/*******************************************************************************
File:			Main.c

Abstract:	Main ATmega8 function library for the wifi radio project

Platform:	ATmega8

Functions:
extern void	main(void);
ISR(SIG_OVERFLOW0)

Versions:	01/14/10	GND	Gary N. Dion
*******************************************************************************/

// OS headers
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <avr/pgmspace.h>

// General purpose include files
#include "StdDefines.h"

// App required include files
#include "Main.h"
#include	"Control.h"
#include "Serial.h"
#include "VFD.h"
#include "Messaging.h"

// Local definitions
#define WatchdogReset() asm("wdr")		// Macro substitution to kick the dog

// Local variables
unsigned char Idle;


/******************************************************************************/
extern int main(void)
/*******************************************************************************
* ABSTRACT:		Main program entry function.
*
* PARAMETERS:	None.
*/
{
	unsigned char schedule = 0;			// Maintains the schedule state

	CtrlRegInit();								// Initialize control Registers
	SerInit();									// Initialize serial communication
	VFDInit();									// Initialize vacuum fluorescent fisplay
	MsgInit();									// Initialize messaging system
	sei();										// Enable interrupts

	//	Initialize the 8-bit Timer0 to clock @ 172.8 kHz with XTAL of 11.0596 MHz
	TCCR0 = 0x03;								// Timer0 control register prescale @ 64
	TIMSK |= 1<<TOIE0;						// Enable Timer0 interrupt

	// Enable the watchdog timer
	WDTCR	= (1<<WDCE) | (1<<WDE);			// Wake-up the watchdog register
	WDTCR	= (1<<WDE) | 1;					// Enable and set timout around 35 ms

	WatchdogReset();							// Reset watchdog

	// Initialization complete - system ready.  Run main program loop!
	while(1)
	{
		CtrlSchedule(schedule);				// Execute scheduled segments
		Idle = TRUE;							// Set idle state flag.

		while(Idle)								// Stay in the loop during idle periods
		{
			SerProcesses();					// Handle TX/RX communications
			CtrlIdle();							// Control willy-nilly processes
		}

		if(++schedule == 100) schedule = 0; // Bump/wrap schedule

		WatchdogReset();						// Reset the watchdog
	}

	return(1);

}		// End main(...)


/******************************************************************************/
ISR(SIG_OVERFLOW0)
/*******************************************************************************
* ABSTRACT:	This function handles the counter0 overflow interrupt.  This timer's
*				counter increments at 172.8 kHz.  Therefore, for the interrupt
*				to occur at 19.2 kHz. We must preload the counter with 256 - 9, or
*				247.  The function then calls the handler for the software UART.
*           The transmit therefore UART runs at 19.2 kHz.  And if we called the
*				receive UART, it would end up running at a speed of 4800 baud
*           (or one quarter of 19.2 kHz).  But since I'm not using it, no call.
*				A debounce routine is then called for input buttons and encoders.
*				Finally, every 192 counts, Idle is reset for the 100 Hz counter.
*
* PARAMETERS:	None.
*/
{
	static unsigned char overflowcount;

	TCNT0 = 247;						//	Preload the counter to interrupt at 19.2 kHz

//	SerSoftUART_RX();					// Take case of the software UART receiver
	SerSoftUART_TX();					// Take case of the software UART transmitter
	CtrlDebounce();					// Take care of button/knob input debouncing
	CtrlRI();							// Take care of infrared commands

	// Clear Idle every 192 calls, or every 0.01 seconds, for 100 Hz main loop.
	if (!(--overflowcount))
	{
		overflowcount = 192;			// Preload counter for 192 decrements
		Idle = FALSE;					// And clear the Idle flag
	}

}		// End interrupt handler
