/*******************************************************************************
File:			Messaging.c

Abstract:	Messaging subsystem function library

Platform:	Any

Functions:
extern void MsgInit(void);
extern void MsgHandler0(unsigned char newchar);

Version:		1.00	01/14/10	GND	Gary N. Dion
*******************************************************************************/

// General purpose include files
#include "StdDefines.h"

// App required include files
#include "Messaging.h"
#include "Serial.h"
#include "Control.h"

// Local definitions
#define HISTORY_SIZE		(6)				// Keep track of a few characters
#define BUFFER_SIZE		(140)				// Maximum size of the text buffer

// Local variables
unsigned char history[HISTORY_SIZE];	// History of the incoming bytes
unsigned char message_type;				// Name, Title, Volum, STREAM, etc.

/******************************************************************************/
extern void MsgInit(void)
/*******************************************************************************
* ABSTRACT:	Initialize all the messaging registers
*
* INPUT:		None.
* OUTPUT:	None.
*/
{
	unsigned char i;							// Generic loop variable

	message_type = NONE;						// Initial state is no message
	for (i = 0; i < HISTORY_SIZE; i++)	// For the history buffer
	{
		history[i] = 0;						// Clear it
	}

	return;

}		// MsgInit(...)


/******************************************************************************/
extern void MsgHandler0(unsigned char newchar)
/*******************************************************************************
* ABSTRACT:	Processes the characters coming in from USART.  In this case,
*				this is the port connected to the "network".
*
* INPUT:		newchar			Next character from the serial port.
* OUTPUT:	None.
*/
{
	static char buffer[BUFFER_SIZE];
	static unsigned char buffer_index = 0;
	unsigned char i;

	for (i = 0; i < HISTORY_SIZE; i++)
	{
		history[i] = history[i + 1];		// Shift the history buffer back one byte
	}

	history[HISTORY_SIZE - 1] = newchar;// And put the new character on the end

	if ((history[HISTORY_SIZE - 1] == ' ') &&
			(history[HISTORY_SIZE - 2] == '#') &&
				(history[HISTORY_SIZE - 3] == '/') &&
					(history[HISTORY_SIZE - 4] == ':') &&
						(history[HISTORY_SIZE - 5] == 't')) // True for "t:/# "
	{
		CtrlProcess(buffer, PROMPT, 0);	// Report we just received a prompt
		buffer_index = 0;						// Clear buffer index
		message_type = NONE;					// Clear message type
		return;									// Exit function
	}

	if ((history[HISTORY_SIZE - 1] == ' ') &&
			(history[HISTORY_SIZE - 2] == ':') &&
				(history[HISTORY_SIZE - 3] == 'e') &&
					(history[HISTORY_SIZE - 4] == 'm') &&
						(history[HISTORY_SIZE - 5] == 'a')) // True for "ame: "
	{
		buffer_index = 0;						// Clear buffer index
		message_type = NAME;					// Start capturing Name next time
		return;									// Exit function
	}

	if ((history[HISTORY_SIZE - 1] == ' ') &&
			(history[HISTORY_SIZE - 2] == ':') &&
				(history[HISTORY_SIZE - 3] == 'e') &&
					(history[HISTORY_SIZE - 4] == 'l') &&
						(history[HISTORY_SIZE - 5] == 't')) // True for "tle: "
	{
		buffer_index = 0;						// Clear buffer index
		message_type = TITLE;				// Start capturing Title next time
		return;									// Exit function
	}

	if ((history[HISTORY_SIZE - 1] == ':') &&
			(history[HISTORY_SIZE - 2] == 'e') &&
				(history[HISTORY_SIZE - 3] == 'm') &&
					(history[HISTORY_SIZE - 4] == 'u') &&
						(history[HISTORY_SIZE - 5] == 'l')) // True for "lume:"
	{
		buffer_index = 0;						// Clear buffer index
		message_type = VOLUM;				// Start capturing Volume next time
		return;									// Exit function
	}

	if ((history[HISTORY_SIZE - 1] == '#') &&
			(history[HISTORY_SIZE - 2] == ' ') &&
				(history[HISTORY_SIZE - 3] == ']') &&
					(history[HISTORY_SIZE - 4] == 'g') &&
						(history[HISTORY_SIZE - 5] == 'n')) // True for "ng] #"
	{
		buffer_index = 0;						// Clear buffer index
		message_type = STREAM;				// Start capturing Stream next time
		return;									// Exit function
	}

	switch (message_type)
	{
		case (NONE):
			break;

		case (NAME):	// Report Name when newchar = '\n' or buffer overflow
			if ((newchar == 0x0D) || ((buffer_index + 1) == BUFFER_SIZE))
			{
				buffer[buffer_index] = 0;	// NULL-terminate the string
				CtrlProcess(buffer, NAME, buffer_index);	// Report Name
				message_type = NONE;			// Clear message type
			}

			buffer[buffer_index++] = newchar;	// Otherwise, just save character
			break;

		case (TITLE):	// Report Title when newchar = '\n' or buffer overflow
			if ((newchar == 0x0D) || ((buffer_index + 1) == BUFFER_SIZE))
			{
				buffer[buffer_index] = 0;	// NULL-terminatee the string
				CtrlProcess(buffer, TITLE, buffer_index);	// Report Title
				message_type = NONE;			// Clear message type
			}

			buffer[buffer_index++] = newchar;	// Otherwise, just save character
			break;

		case (VOLUM):	// Report Volume when newchar = '%' or buffer overflow
			if ((newchar == '%') || ((buffer_index + 1) == BUFFER_SIZE))
			{
				buffer[buffer_index] = 0;	// NULL-terminate the string
				CtrlProcess(buffer, VOLUM, buffer_index);	// Report Volume
				message_type = NONE;			// Clear message type
			}

			buffer[buffer_index++] = newchar;	// Otherwise, just save character
			break;

		case (STREAM):	// Report Stream on ' ' or buffer overflow
			if ((newchar == ' ') || ((buffer_index + 1) == BUFFER_SIZE))
			{
				buffer[buffer_index] = 0;	// NULL-terminate the string
				CtrlProcess(buffer, STREAM, buffer_index);	// Report Stream
				buffer_index = 0;				// Reset the buffer index
				message_type = ELAPSED;		// Next, capture Elapsed time
			}

			buffer[buffer_index++] = newchar;	// Otherwise, just save character
			break;

		case (ELAPSED):	// Report Elapsed time on '/' or buffer overflow
			if ((newchar == '/') || ((buffer_index + 1) == BUFFER_SIZE))
			{
				buffer[buffer_index] = 0;	// NULL-terminate the string
				CtrlProcess(buffer, ELAPSED, buffer_index);	// Report Elapsed time
				message_type = NONE;			// Clear message type
			}

			if (newchar != ' ')	buffer[buffer_index++] = newchar; // Skip spaces
			break;
	}

	return;

} 		// End MsgHandler0(...)
