/*******************************************************************************
File:			Serial.c

Abstract:	Serial I/O function library.

Platform:	ATmega8

Functions:
extern void	SerInit(void);
extern void	SerSendByte(unsigned char chr, char port);
extern void SerSendString(unsigned char *address, char chars, char port);
extern void SerSendFLASHString(const char *addr, char chars, char port);
extern void SerSendEEPROMString(unsigned short addr, char chars, char port);
extern void SerProcesses (void);

SIGNAL(SIG_UART_RECV)
SIGNAL(SIG_UART_TRANS)

Version:		1.00	01/14/10	GND	Gary N. Dion
*******************************************************************************/

// OS headers
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <avr/pgmspace.h>

// General purpose include files
#include "StdDefines.h"

// App required include files
#include "Serial.h"
#include "Messaging.h"

// Local variables
unsigned char in0buf[RX_BUF_SIZE0];		// USART input buffer
unsigned char in0head;						// USART input buffer head pointer
unsigned char in0tail;						// USART input buffer tail pointer
unsigned char out0buf[TX_BUF_SIZE0];	// USART output buffer
unsigned char out0head;						// USART output buffer head pointer
unsigned char out0tail;						// USART output buffer tail pointer

#ifdef SOFTUART
	unsigned char in1buf[RX_BUF_SIZE1];	// SoftUART input buffer
	unsigned char in1head;					// SoftUART input buffer head pointer
	unsigned char in1tail;					// SoftUART input buffer tail pointer
	unsigned char out1buf[TX_BUF_SIZE1];// SoftUART output buffer
	unsigned char out1head;					// SoftUART output buffer head pointer
	unsigned char out1tail;					// SoftUART output buffer tail pointer
	// The data directions for the following pins get defined in Control.c
	#define	RXPIN			(PIND & 0x20)		// Port D, pin 5 is the RX pin
	#define	TXPINLOW		(PORTD &= ~0x40)	// Port D, pin 6 is the TX pin
	#define	TXPINHIGH	(PORTD |= 0x40)	// Port D, pin 6 is the TX pin
#endif


/******************************************************************************/
extern void	SerInit(void)
/*******************************************************************************
* ABSTRACT:	This function initializes the serial USART.
*				It establishes the baud rate, data and stop bits, and parity.
*
* INPUT:		None
* RETURN:	Void
*/
{
	// Set baud rate
	UBRRH = 0;
//	UBRRL = 5;  								// Equates to 115.2k baud at 11.0592 MHz
//	UBRRL = 17;  								// Equates to 38.4k baud at 11.0592 MHz
	UBRRL = 35;  								// Equates to 19.2k baud at 11.0592 MHz
// UBRRL = 71;  								// Equates to 9600 baud at 11.0592 MHz
//	UBRRL = 143;  								// Equates to 4800 baud at 11.0592 MHz

	// Set frame format to 8 data bits, no parity, and 1 stop bit
	UCSRC = (1<<URSEL)|(3<<UCSZ0);

	// Enable the receiver, the transmitter, and their respective interrupts
	UCSRB = (1<<RXEN)|(1<<TXEN)|(1<<RXCIE)|(1<<TXCIE);

	in0head = in0tail = out0head = out0tail = 0;	// Clear all the pointers
#ifdef SOFTUART
	in1head = in1tail = out1head = out1tail = 0;	// Clear all the pointers
#endif

	return;

}		// End SerInit(...)


/******************************************************************************/
extern void	SerSendByte(unsigned char chr, char port)
/*******************************************************************************
* ABSTRACT:	This function pushes a new character into the output buffer,
*				then pre-advances the pointer to the next empty location.
*
* INPUT:		chr	byte to send
*				port	0 is the USART, 1 is the Software UART
* RETURN:	Void
*/
{
	if (port == 0)
	{
		if (++out0head == TX_BUF_SIZE0) out0head = 0;	// Advance/wrap pointer
		out0buf[out0head] = chr;		 	// Transfer the byte to output buffer

		if (UCSRA & (1<<UDRE))				// If the transmit buffer is empty
		{
			if (++out0tail == TX_BUF_SIZE0) out0tail = 0;// Advance/wrap pointer
			UDR = out0buf[out0tail];		// Place the byte in the output buffer
		}
	}
#ifdef SOFTUART
	else
	{
		if (++out1head == TX_BUF_SIZE1) out1head = 0;	// Advance/wrap pointer
		out1buf[out1head] = chr;		 	// Transfer the byte to output buffer
	}
#endif

	return;

}		// End SendByte(...)


/******************************************************************************/
extern void SerSendString(unsigned char *address, char chars, char port)
/*******************************************************************************
* ABSTRACT:	This function sends a null-terminated string to the serial port.
*
* INPUT:		*address	Pointer to string to send
*				chars		Maximum number of characters to send
*				port		0 is the USART, 1 is the Software UART
* RETURN:	Void
*/
{
	while ((*address != 0) && (chars--))// While not end of string
	{
		SerSendByte(*(address++),port);	// Send the byte and increment index
	}

	return;

}		// End SerSendString(...)


/******************************************************************************/
extern void SerSendFLASHString(const char *addr, char chars, char port)
/*******************************************************************************
* ABSTRACT:	This function sends a null-terminated string located in program
*				space to the serial port.
*
* INPUT:		addr		Starting address for the string
*				chars		Maximum number of characters to send
*				port		0 is the USART, 1 is the Software UART
* RETURN:	Void
*/
{
	unsigned char temp_char;

	temp_char = pgm_read_byte(addr++);	// Preload first character
	while ((temp_char) && (chars--))		// While not end of string
	{
		SerSendByte(temp_char, port);		// Send byte to desired port
		temp_char = pgm_read_byte(addr++);	// Grab next byte
	}

	return;

}		// End SerSendFLASHString(...)


/******************************************************************************/
extern void SerSendEEPROMString(unsigned short addr, char chars, char port)
/*******************************************************************************
* ABSTRACT:	This function sends a null-terminated string found in EEPROM
*				at the address given to the serial port.
*
* INPUT:		addr		Starting address for the string
*				chars		Maximum number of characters to send
*				port		0 is the USART, 1 is the Software UART
* RETURN:	Void
*/
{
	unsigned char temp_char;

	temp_char = eeprom_read_byte ((uint8_t *)(addr));	// Preload first character
	while ((temp_char) && (chars--))		// While not end of string
	{
		SerSendByte(temp_char, port);		// Send byte to desired port
		temp_char = eeprom_read_byte ((uint8_t *)(++addr));	// Grab next byte
	}

	return;

}		// End SerSendEEPROMString(...)


/******************************************************************************/
extern void SerProcesses(void)
/*******************************************************************************
* ABSTRACT:	Called by main.c during idle time. Processes any waiting serial
*				characters coming in either serial ports.
*
* INPUT:		None
* RETURN:	Void
*/
{
	if (in0tail != in0head)					// If incoming bytes are pending
	{
		if (++in0tail == RX_BUF_SIZE0) in0tail = 0;// Advance/wrap pointer
//		SerSendByte(in0buf[in0tail]+1,0);	// Echo character for debug
		MsgHandler0(in0buf[in0tail]);		// And pass it to a handler
	}

#ifdef SOFTUART
	if (in1tail != in1head)					// If incoming bytes are pending
	{
		if (++in1tail == RX_BUF_SIZE1) in1tail = 0;// Advance/wrap pointer
//		SerSendByte(in1buf[in1tail],1);	// Echo character for debug
//		MsgHandler1(in1buf[in1tail]);		// And pass it to a handler
	}
#endif

	return;

}		// End Serial_Processes(...)


#ifdef SOFTUART
/******************************************************************************/
extern void	SerSoftUART_TX(void)
/*******************************************************************************
* ABSTRACT:	This function is called at the desired baud rate for the transmit
*				port.  This function effectively creates a software UART using the
*				pins declared at the top of this file.  Protocol is 8N1.
*
* INPUT:		None
* RETURN:	Void
*/
{
	static unsigned char txbit = 10;
	static unsigned char tx_byte;

	// This section sees if the transmitter is idle and a tx byte is pending
	if ((txbit == 10) && (out1tail != out1head))
	{
		if (++out1tail == TX_BUF_SIZE1) out1tail = 0;// Advance/wrap pointer

		tx_byte = out1buf[out1tail];		// Take the byte from the output buffer
		txbit = 0;								// Set bit index to zero
	}

	// And this section runs as the byte is being transmitted
	if (txbit != 10)							// Run for each pending bit
	{
		if (!txbit) TXPINLOW;				// First bit is the start bit
		else
		{
			if (txbit != 9)					// For each of the 8-bits in the byte
			{
				if (tx_byte & 1) TXPINHIGH;// If bit is high, pin is high
				else TXPINLOW;					// else, pin is low

				tx_byte >>= 1;					// Shift byte right for next bit
			}

			else TXPINHIGH;					// Send a high for the stop bit
		}

		txbit++;									// Increment bit count for next time
	}

	return;

}		// End SerSoftUART_TX(...)


/******************************************************************************/
extern void	SerSoftUART_RX(void)
/*******************************************************************************
* ABSTRACT:	This function must be called at a rate of 4 times the desired baud
*				rate.  This function effectively creates a software UART using the
*				pins declared at the top of this file.  Protocol is 8N1.
*
* INPUT:		None
* RETURN:	Void
*/
{
	static unsigned char Receiving = 0;
	static unsigned char rxbit;
	static unsigned char rxclock;
	static unsigned char rx_byte;

	if (!Receiving)							// Not receiving?  Seeking the start bit
	{
		if (!(RXPIN)) 							// A low value may be a start bit
		{
			if (rxclock == 0)					// Cleared by previous start loop
			{
				Receiving = 1;					// Advance on confirming start bit
				rx_byte = 0;					// Begin with a clean slate
				rxbit = 0;						// Bit index is lsb first
			}

			rxclock = 0;						// Always reset the receive clock
		}

	}
	else
	{
		if (!(++rxclock % 4))				// Execute code every 4th rx clock tick
		{
			if (rxbit != 8)					// Only grab eight data bits
			{
				if (RXPIN) rx_byte |= 1<<rxbit;	// Stuff next bit receive byte
				rxbit++;							// Advance bit count
			}
			else									// We have eight bits now
			{
				if (RXPIN)						// Make sure bit is high for stop bit
				{
					if (++in1head == RX_BUF_SIZE1) in1head = 1;// Advance & wrap
					in1buf[in1head] = rx_byte;	// Place byte in receive buffer
				}

				Receiving = 0;					// Revert to looking for next start bit
			}

		}

	}

}		// End SerSoftUART_RX(...)

#endif


/******************************************************************************/
ISR(SIG_UART_RECV)
/*******************************************************************************
* ABSTRACT:	Called by the receive ISR (interrupt). Saves the next serial byte
*				to the head of the RX buffer.
*/
{
	if (++in0head == RX_BUF_SIZE0) in0head = 0;	// Advance/wrap pointer

	in0buf[in0head] = UDR;	  				// Transfer the byte to the input buffer

}		// End SIGNAL(SIG_UART_RECV)


/******************************************************************************/
ISR(SIG_UART_TRANS)
/*******************************************************************************
* ABSTRACT:	Called by the transmit ISR (interrupt). Puts the next serial byte
*				into the TX register.
*/
{
	if (out0tail != out0head)				// If there are outgoing bytes pending
	{
		if (++out0tail == TX_BUF_SIZE0) out0tail = 0;	// Advance/wrap pointer

		UDR = out0buf[out0tail];			// Place the byte in the output buffer
	}

}		// End SIGNAL(SIG_UART_TRANS)
