/*******************************************************************************
File:			serial.c

Abstract:	VFD subsystem function library.

Platform:	ATmega8

Functions:
extern void	VFDInit(void);
extern void VFDClear(void);
extern void VFDRedraw(void);
extern void VFDCursor(char position, char enable, char blink);
extern void VFDsendString(char *addr, unsigned char position, char chars);
extern void VFDsendFLASHString(char *addr, unsigned char position, char chars);
extern void VFDsendEEPROMString(char *addr, unsigned char position, char chars);

Versions:	1.00	01/14/10	GND	Gary N. Dion
*******************************************************************************/

// OS headers
#include <avr/pgmspace.h>
#include <avr/eeprom.h>

// General purpose include files
#include "StdDefines.h"

// App required include files
#include "VFD.h"
#include "Serial.h"

// Local variables
unsigned char cursor_pos;					// Position of the cursor on the screen
unsigned char VFD_array[80];				// Display RAM, refreshed on command


/******************************************************************************/
extern void	VFDInit(void)
/*******************************************************************************
* ABSTRACT:	This function initializes the VFD controller(s).
*
* INPUT:		None
* RETURN:	None
*/
{
	SerSendByte(0x0E,1);						// Cursor Off
	SerSendByte(0x15,1);						// Clear display and home cursor

	VFDClear();

	return;

}		// End VFDInit(...)


/******************************************************************************/
extern void VFDClear(void)
/*******************************************************************************
* ABSTRACT:	Set cursor to new position
*
* INPUT:		position	position to place the cursor
* RETURN:	None
*/
{
	unsigned char i;

	for (i = 0; i < 80; i++)
	{
		VFD_array[i] = ' ';					// Fill all 80 characters of display
	}

	return;

}		// End VFDCursor(...)


/******************************************************************************/
extern void VFDRedraw(void)
/*******************************************************************************
* ABSTRACT:	This function simply initiates a screen redraw by setting a flag.
*
* INPUT:		None
* OUTPUT:	None
* RETURN:	None
*/
{
	unsigned char i;

	for (i = 0; i < 80; i++)
	{
		if (VFD_array[i] > 0x1F)
			SerSendByte(VFD_array[i], 1);	// Write all 80 characters of display
	}
	SerSendByte(0x16,1);						// Home cursor

	return;
}		// End VFDRedraw(...)


/******************************************************************************/
extern void VFDDim(char brightness)
/*******************************************************************************
* ABSTRACT:	This function controls the brightness of the display.
*
* INPUT:		brightness
* RETURN:	None
*/
{
	SerSendByte(0x19,1);						// Set address bit 0 high
	SerSendByte(0x30,1);						// Set display screen brightness level
	SerSendByte(0xFF,1);						// For all comumns
	SerSendByte(7 - brightness,1);				// to desired value

	return;

}		// End VFDCursor(...)


/******************************************************************************/
extern void VFDCursor(unsigned char position)
/*******************************************************************************
* ABSTRACT:	Set cursor to new position
*
* INPUT:		position	position to place the cursor
* RETURN:	None
*/
{
	cursor_pos = position;					// Set cursor to new position

	return;

}		// End VFDCursor(...)


/******************************************************************************/
extern void VFDChar(char newchar)
/*******************************************************************************
* ABSTRACT:	This function sends one character to the VFD buffer
*
* INPUT:		newchar	Character to display at 'cursor_pos'
* RETURN:	None
*/
{
	if (cursor_pos > 79) cursor_pos = 79;	// No buffer overflow!
	VFD_array[cursor_pos++] = newchar;		// Place character

	return;

}		// End VFDsendString(...)

/******************************************************************************/
extern void VFDsendString(char *addr, unsigned char position, char chars)
/*******************************************************************************
* ABSTRACT:	This function sends a null-terminated string to the VFD buffer
*				starting at coordinates x,y.
*
* INPUT:		address	Starting address for the string
*				position	Starting position on the display for the string
*				chars		Maximum number of characters to display
* RETURN:	None
*/
{
	cursor_pos = position;
	while ((*addr != 0) && (chars--))
	{
		if (cursor_pos > 79) cursor_pos = 79;	// No buffer overflow!
		VFD_array[cursor_pos++] = (*addr);
		addr++;
	}

	return;

}		// End VFDsendString(...)


/******************************************************************************/
extern void VFDsendFLASHString(const char *addr,
													unsigned char position, char chars)
/*******************************************************************************
* ABSTRACT:	This function sends a null-terminated string located in program
*				space to the VFD buffer starting at coordinates x,y.
*
* INPUT:		address	Starting address for the string
*				position	Starting position on the display for the string
*				chars		Maximum number of characters to display
* RETURN:	None
*/
{
	unsigned char temp_char;

	cursor_pos = position;
	temp_char = pgm_read_byte(addr++);
	while ((temp_char) && (chars--))
	{
		if (cursor_pos > 79) cursor_pos = 79;	// No buffer overflow!
		VFD_array[cursor_pos++] = temp_char;
		temp_char = pgm_read_byte(addr++);
	}

	return;

}		// End VFDsendString_P(...)


/******************************************************************************/
extern void VFDsendEEPROMString(unsigned int addr,
													unsigned char position, char chars)
/*******************************************************************************
* ABSTRACT:	This function sends a null-terminated string found in EEPROM
*				at the address given to the VFD, starting at coordinates x,y.
*
* INPUT:		address	Starting address for the string
*				position	Starting position on the display for the string
*				chars		Maximum number of characters to display
* RETURN:	None
*/
{
	unsigned char temp_char;

	cursor_pos = position;
	temp_char = eeprom_read_byte ((uint8_t *)(addr));
	while ((temp_char) && (chars--))
	{
		if (cursor_pos > 79) cursor_pos = 79;	// No buffer overflow!
		VFD_array[cursor_pos++] = temp_char;
		temp_char = eeprom_read_byte ((uint8_t *)(++addr));
	}

	return;

}		// End VFDsendEEPROMString(...)

